@@ -80,8 +80,8 @@ namespace OpenBve {
 			try {
 				LoadEverythingThreaded();
 			} catch (Exception ex) {
-				if (PluginManager.PluginError) {
-					Interface.AddMessage(Interface.MessageType.Critical, false, "The plugin " + PluginManager.PluginName + " caused a critical error in the route and train loader: " + ex.Message);
+				if (PluginManager.CurrentPlugin != null && PluginManager.CurrentPlugin.LastException != null) {
+					Interface.AddMessage(Interface.MessageType.Critical, false, "The train plugin " + PluginManager.CurrentPlugin.PluginTitle + " caused a critical error in the route and train loader: " + PluginManager.CurrentPlugin.LastException.Message);
 				} else {
 					Interface.AddMessage(Interface.MessageType.Critical, false, "The route and train loader encountered the following critical error: " + ex.Message);
 				}
@@ -129,12 +129,6 @@ namespace OpenBve {
 				}
 			}
 			RouteProgress = 1.0;
-			// camera
-			ObjectManager.InitializeVisibility();
-			TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, 0.0, true, false);
-			TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, -0.1, true, false);
-			TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, 0.1, true, false);
-			World.CameraTrackFollower.TriggerType = TrackManager.EventTriggerType.Camera;
 			// initialize trains
 			System.Threading.Thread.Sleep(1); if (Cancel) return;
 			TrainManager.Trains = new TrainManager.Train[Game.PrecedingTrainTimeDeltas.Length + 1 + (Game.BogusPretrainInstructions.Length != 0 ? 1 : 0)];
@@ -302,146 +296,20 @@ namespace OpenBve {
 			// finished created objects
 			System.Threading.Thread.Sleep(1); if (Cancel) return;
 			ObjectManager.FinishCreatingObjects();
-			// starting time and track position
-			System.Threading.Thread.Sleep(1); if (Cancel) return;
-			Game.SecondsSinceMidnight = 0.0;
-			Game.StartupTime = 0.0;
-			int PlayerFirstStationIndex = -1;
-			double PlayerFirstStationPosition = 0.0;
-			for (int i = 0; i < Game.Stations.Length; i++) {
-				if (Game.Stations[i].StopMode == Game.StationStopMode.AllStop | Game.Stations[i].StopMode == Game.StationStopMode.PlayerStop & Game.Stations[i].Stops.Length != 0) {
-					PlayerFirstStationIndex = i;
-					int s = Game.GetStopIndex(i, TrainManager.PlayerTrain.Cars.Length);
-					if (s >= 0) {
-						PlayerFirstStationPosition = Game.Stations[i].Stops[s].TrackPosition;
-					} else {
-						PlayerFirstStationPosition = Game.Stations[i].DefaultTrackPosition;
-					}
-					if (Game.Stations[i].ArrivalTime < 0.0) {
-						if (Game.Stations[i].DepartureTime < 0.0) {
-							Game.SecondsSinceMidnight = 0.0;
-							Game.StartupTime = 0.0;
-						} else {
-							Game.SecondsSinceMidnight = Game.Stations[i].DepartureTime - Game.Stations[i].StopTime;
-							Game.StartupTime = Game.Stations[i].DepartureTime - Game.Stations[i].StopTime;
-						}
-					} else {
-						Game.SecondsSinceMidnight = Game.Stations[i].ArrivalTime;
-						Game.StartupTime = Game.Stations[i].ArrivalTime;
-					}
-					break;
-				}
-			}
-			int OtherFirstStationIndex = -1;
-			double OtherFirstStationPosition = 0.0;
-			double OtherFirstStationTime = 0.0;
-			for (int i = 0; i < Game.Stations.Length; i++) {
-				if (Game.Stations[i].StopMode == Game.StationStopMode.AllStop | Game.Stations[i].StopMode == Game.StationStopMode.PlayerPass & Game.Stations[i].Stops.Length != 0) {
-					OtherFirstStationIndex = i;
-					int s = Game.GetStopIndex(i, TrainManager.PlayerTrain.Cars.Length);
-					if (s >= 0) {
-						OtherFirstStationPosition = Game.Stations[i].Stops[s].TrackPosition;
-					} else {
-						OtherFirstStationPosition = Game.Stations[i].DefaultTrackPosition;
-					}
-					if (Game.Stations[i].ArrivalTime < 0.0) {
-						if (Game.Stations[i].DepartureTime < 0.0) {
-							OtherFirstStationTime = 0.0;
-						} else {
-							OtherFirstStationTime = Game.Stations[i].DepartureTime - Game.Stations[i].StopTime;
-						}
-					} else {
-						OtherFirstStationTime = Game.Stations[i].ArrivalTime;
-					}
-					break;
-				}
-			}
-			if (Game.PrecedingTrainTimeDeltas.Length != 0) {
-				OtherFirstStationTime -= Game.PrecedingTrainTimeDeltas[Game.PrecedingTrainTimeDeltas.Length - 1];
-				if (OtherFirstStationTime < Game.SecondsSinceMidnight) {
-					Game.SecondsSinceMidnight = OtherFirstStationTime;
-				}
-			}
-			// score
-			System.Threading.Thread.Sleep(1); if (Cancel) return;
-			Game.CurrentScore.ArrivalStation = PlayerFirstStationIndex + 1;
-			Game.CurrentScore.DepartureStation = PlayerFirstStationIndex;
-			Game.CurrentScore.Maximum = 0;
-			for (int i = 0; i < Game.Stations.Length; i++) {
-				if (i != PlayerFirstStationIndex & Game.PlayerStopsAtStation(i)) {
-					Game.CurrentScore.Maximum += Game.ScoreValueStationArrival;
-				}
-			}
-			if (Game.CurrentScore.Maximum <= 0) {
-				Game.CurrentScore.Maximum = Game.ScoreValueStationArrival;
-			}
-			// initialize trains
-			System.Threading.Thread.Sleep(1); if (Cancel) return;
-			for (int i = 0; i < TrainManager.Trains.Length; i++) {
-				TrainManager.InitializeTrain(TrainManager.Trains[i]);
-				int s = i == TrainManager.PlayerTrain.TrainIndex ? PlayerFirstStationIndex : OtherFirstStationIndex;
-				if (s >= 0) {
-					if (Game.Stations[s].OpenLeftDoors) {
-						for (int j = 0; j < TrainManager.Trains[i].Cars.Length; j++) {
-							TrainManager.Trains[i].Cars[j].Specs.AnticipatedLeftDoorsOpened = true;
-						}
-					}
-					if (Game.Stations[s].OpenRightDoors) {
-						for (int j = 0; j < TrainManager.Trains[i].Cars.Length; j++) {
-							TrainManager.Trains[i].Cars[j].Specs.AnticipatedRightDoorsOpened = true;
-						}
-					}
-				}
-				if (Game.Sections.Length != 0) {
-					Game.Sections[0].Enter(TrainManager.Trains[i]);
-				}
-				for (int j = 0; j < TrainManager.Trains[i].Cars.Length; j++) {
-					double length = TrainManager.Trains[i].Cars[0].Length;
-					TrainManager.MoveCar(TrainManager.Trains[i], j, -length, 0.01);
-					TrainManager.MoveCar(TrainManager.Trains[i], j, length, 0.01);
-				}
-			}
-			// signals
-			System.Threading.Thread.Sleep(1); if (Cancel) return;
+			// update sections
 			if (Game.Sections.Length > 0) {
 				Game.UpdateSection(Game.Sections.Length - 1);
 			}
 			// load plugin
-			System.Threading.Thread.Sleep(1); if (Cancel) return;
 			if (TrainManager.PlayerTrain != null) {
-				PluginManager.LoadAtsConfig(CurrentTrainFolder, CurrentTrainEncoding, TrainManager.PlayerTrain);
+				PluginManager.LoadPlugin(CurrentTrainFolder, CurrentTrainEncoding, TrainManager.PlayerTrain);
 			}
-			// move train in position
-			for (int i = 0; i < TrainManager.Trains.Length; i++) {
-				double p;
-				if (i == TrainManager.PlayerTrain.TrainIndex) {
-					p = PlayerFirstStationPosition;
-				} else if (TrainManager.Trains[i].State == TrainManager.TrainState.Bogus) {
-					p = Game.BogusPretrainInstructions[0].TrackPosition;
-					TrainManager.Trains[i].AI = new Game.BogusPretrainAI(TrainManager.Trains[i]);
-				} else {
-					p = OtherFirstStationPosition;
-				}
-				for (int j = 0; j < TrainManager.Trains[i].Cars.Length; j++) {
-					TrainManager.MoveCar(TrainManager.Trains[i], j, p, 0.01);
-				}
-			}
-			// timetable
-			System.Threading.Thread.Sleep(1); if (Cancel) return;
-			if (Timetable.DefaultTimetableDescription.Length == 0) {
-				Timetable.DefaultTimetableDescription = Game.LogTrainName;
-			}
-			// initialize camera
-			System.Threading.Thread.Sleep(1); if (Cancel) return;
-			if (World.CameraRestriction == World.CameraRestrictionMode.NotAvailable) {
-				World.CameraMode = World.CameraViewMode.InteriorLookAhead;
-			}
-			TrainManager.UpdateCamera(TrainManager.PlayerTrain);
-			TrackManager.UpdateTrackFollower(ref World.CameraTrackFollower, -1.0, true, false);
-			ObjectManager.UpdateVisibility(World.CameraTrackFollower.TrackPosition + World.CameraCurrentAlignment.Position.Z);
-			World.CameraSavedInterior = new World.CameraAlignment();
-			World.CameraSavedExterior = new World.CameraAlignment(new World.Vector3D(-2.5, 1.5, -15.0), 0.3, -0.2, 0.0, PlayerFirstStationPosition, 1.0);
-			World.CameraSavedTrack = new World.CameraAlignment(new World.Vector3D(-3.0, 2.5, 0.0), 0.3, 0.0, 0.0, TrainManager.PlayerTrain.Cars[0].FrontAxle.Follower.TrackPosition - 10.0, 1.0);
+			// continue after opengl is initialized
+			/*
+			 * The rest of the loading procedure is located in MainLoop.cs :: StartLoop.
+			 * This is due to the fact that the OpenGL window has not been initialized yet,
+			 * but the follow-up initialization code needs a valid OpenGL context.
+			 * */
 		}
 
 	}
