@@ -957,7 +957,7 @@ namespace OpenBve {
 			Object.Mesh.Materials[0].TransparentColor = new World.ColorRGB(0, 0, 255);
 			Object.Mesh.Materials[0].DaytimeTextureIndex = DaytimeTextureIndex;
 			Object.Mesh.Materials[0].NighttimeTextureIndex = NighttimeTextureIndex;
-			Object.Dynamic = 1;
+			Object.Dynamic = true;
 			// calculate offset
 			World.Vector3D o;
 			o.X = xm + DriverX;
