@@ -1,10 +1,13 @@
 ﻿using System;
-using Tao.OpenGl;
-using Tao.Sdl;
+using System.Diagnostics;
+using System.Reflection;
 using System.Windows.Forms;
 
+using Tao.OpenGl;
+using Tao.Sdl;
+
 namespace OpenBve {
-	internal static partial class Program {
+	public static partial class Program {
 
 		// system
 		internal static string RestartProcessArguments = null;
@@ -18,7 +21,7 @@ namespace OpenBve {
 
 		// main
 		[STAThread]
-		internal static void Main(string[] Args) {
+		public static void Main(string[] args) {
 			Application.EnableVisualStyles();
 			Application.SetCompatibleTextRenderingDefault(false);
 			// platform and mono
@@ -36,8 +39,8 @@ namespace OpenBve {
 			CurrentlyRunOnMono = Type.GetType("Mono.Runtime") != null;
 			// file hierarchy standard
 			if (CurrentPlatform != Platform.Windows) {
-				for (int i = 0; i < Args.Length; i++) {
-					if (Args[i].Equals("/fhs", StringComparison.OrdinalIgnoreCase)) {
+				for (int i = 0; i < args.Length; i++) {
+					if (args[i].Equals("/fhs", StringComparison.OrdinalIgnoreCase)) {
 						UseFilesystemHierarchyStandard = true;
 						break;
 					}
@@ -51,16 +54,17 @@ namespace OpenBve {
 			}
 			// start
 			#if DEBUG
-			Start(Args);
+			Start(args);
 			#else
 			try {
-				Start(Args);
+				Start(args);
 			} catch (Exception ex) {
-				string t = GetExceptionText(ex, 5);
-				if (PluginManager.PluginError & PluginManager.PluginName != null) {
-					MessageBox.Show("The " + PluginManager.PluginName + " plugin raised the following exception:\n\n" + t, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
+				if (PluginManager.CurrentPlugin != null && PluginManager.CurrentPlugin.LastException != null) {
+					string text = GetExceptionText(PluginManager.CurrentPlugin.LastException, 5);
+					MessageBox.Show("The train plugin " + PluginManager.CurrentPlugin.PluginTitle + " caused the following exception:\n\n" + text, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
 				} else {
-					MessageBox.Show(t, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
+					string text = GetExceptionText(ex, 5);
+					MessageBox.Show(text, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
 				}
 			}
 			#endif
@@ -81,11 +85,11 @@ namespace OpenBve {
 			Sdl.SDL_Quit();
 			// restart
 			if (RestartProcessArguments != null) {
-				System.Reflection.Assembly Assembly = System.Reflection.Assembly.GetExecutingAssembly();
+				System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
 				if (Program.UseFilesystemHierarchyStandard) {
 					RestartProcessArguments += " /fhs";
 				}
-				System.Diagnostics.Process.Start(Assembly.Location, RestartProcessArguments);
+				System.Diagnostics.Process.Start(assembly.Location, RestartProcessArguments);
 			}
 		}
 
@@ -242,7 +246,7 @@ namespace OpenBve {
 			World.HorizontalViewingAngle = 2.0 * Math.Atan(Math.Tan(0.5 * World.VerticalViewingAngle) * World.AspectRatio);
 			World.OriginalVerticalViewingAngle = World.VerticalViewingAngle;
 			World.ExtraViewingDistance = 50.0;
-			World.ForwardViewingDistance = (double)Interface.CurrentOptions.ViewingDistance + World.ExtraViewingDistance;
+			World.ForwardViewingDistance = (double)Interface.CurrentOptions.ViewingDistance;
 			World.BackwardViewingDistance = 0.0;
 			World.BackgroundImageDistance = (double)Interface.CurrentOptions.ViewingDistance;
 			// load route and train
