@@ -306,12 +306,6 @@ namespace OpenBve {
 								Train.CurrentSectionLimit = double.PositiveInfinity;
 							}
 							Train.CurrentSectionIndex = this.NextSectionIndex;
-							if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) {
-								int a = Game.Sections[this.NextSectionIndex].CurrentAspect;
-								if (a >= 0) {
-									PluginManager.UpdateSignal(Game.Sections[this.NextSectionIndex].Aspects[a].Number);
-								}
-							}
 						}
 					} else {
 						Train.CurrentSectionLimit = double.PositiveInfinity;
@@ -348,21 +342,8 @@ namespace OpenBve {
 					// update train
 					if (this.PreviousSectionIndex >= 0) {
 						if (!Game.Sections[this.PreviousSectionIndex].Invisible) {
-							if (Game.Sections[this.PreviousSectionIndex].CurrentAspect >= 0) {
-								Train.CurrentSectionLimit = Game.Sections[this.PreviousSectionIndex].Aspects[Game.Sections[this.PreviousSectionIndex].CurrentAspect].Speed;
-							} else {
-								Train.CurrentSectionLimit = double.PositiveInfinity;
-							}
 							Train.CurrentSectionIndex = this.PreviousSectionIndex;
 						}
-						if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) {
-							int a = Game.Sections[this.PreviousSectionIndex].CurrentAspect;
-							if (a >= 0) {
-								PluginManager.UpdateSignal(Game.Sections[this.PreviousSectionIndex].Aspects[a].Number);
-							} else {
-								PluginManager.UpdateSignal(255);
-							}
-						}
 					} else {
 						Train.CurrentSectionLimit = double.PositiveInfinity;
 						Train.CurrentSectionIndex = -1;
@@ -382,17 +363,18 @@ namespace OpenBve {
 			}
 		}
 		// transponder
+		/// <summary>Represents the type of transponder. Numerical values less than zero are reserved for built-in functionality. Plugins may only be fed with non-negative types.</summary>
 		internal enum TransponderType {
 			None = -1,
-			S = 0,
-			Sn = 1,
+			SLong = 0,
+			SN = 1,
 			AccidentalDeparture = 2,
 			AtsPPatternOrigin = 3,
 			AtsPImmediateStop = 4,
 			AtsPTemporarySpeedRestriction = -2,
 			AtsPPermanentSpeedRestriction = -3
 		}
-		internal enum TransponderSpecialSection : int {
+		internal enum TransponderSpecialSection {
 			NextRedSection = -2,
 		}
 		internal class TransponderEvent : GeneralEvent {
@@ -445,7 +427,9 @@ namespace OpenBve {
 					Data.OptionalFloat = this.OptionalFloat;
 					Data.OptionalInteger = this.OptionalInteger;
 					if (Train.Specs.Safety.Mode == TrainManager.SafetySystem.Plugin) {
-						PluginManager.UpdateBeacon(Train, Data);
+						if ((int)Data.Type >= 0) {
+							PluginManager.CurrentPlugin.UpdateBeacon((int)Data.Type, Data.SectionIndex, Data.OptionalInteger);
+						}
 					} else {
 						Train.Specs.Safety.AddPendingTransponder(Data);
 					}
@@ -881,6 +865,9 @@ namespace OpenBve {
 				} else if (Direction > 0) {
 					for (int j = 0; j < CurrentTrack.Elements[ElementIndex].Events.Length; j++) {
 						if (OldDelta < CurrentTrack.Elements[ElementIndex].Events[j].TrackPositionDelta & NewDelta >= CurrentTrack.Elements[ElementIndex].Events[j].TrackPositionDelta) {
+							if (CurrentTrack.Elements[ElementIndex].Events[j] is SectionChangeEvent) {
+								SectionChangeEvent section = (SectionChangeEvent)CurrentTrack.Elements[ElementIndex].Events[j];
+							}
 							TryTriggerEvent(CurrentTrack.Elements[ElementIndex].Events[j], 1, Follower.TriggerType, Follower.Train, Follower.CarIndex);
 						}
 					}
